import matplotlib.pyplot as plt

from .._place_holder import Figure_
from .._place_holder import Chart_


class Chart(Chart_):
    """Class containing everything in order to create a complete chart on a
    figure.

    Methods:
    ========

    Set*
    ----

        *   ``setAxisX``
        *   ``setAxisXTimeScale``
        *   ``setAxisXAngularScale``
        *   ``setAxisXLogScale``

        *   ``setAxisY``
        *   ``setAxisYSecondAxis``
        *   ``setAxisYAngularScale``
        *   ``setAxisYLogScale``

        *   ``setTitle``
        *   ``setLegend``
        *   ``setTitle``

    Plot*
    -----

        *   ``plotStandard``
        *   ``plotStandardSecondAxis``

    """

    def __init__(self, fig: Figure_,
                 subPlot: plt.Axes
                 ) -> None:
        super().__init__(fig, subPlot)


    from ._axes_methods import (
        setAxisX,
        setAxisXAngularScale,
        setAxisXTimeScale,
        setAxisXLogScale
    )
    from ._axes_methods import (
        setAxisY,
        setAxisYAngularScale,
        setAxisYLogScale,
        setAxisYSecondAxis
    )

    from ._chart_methods import (
        setBorders, setGrid, setLegend, setTitle
    )

    from ._plot_methods import plotStandard
    from ._plot_methods import plotCbar, plotPcolor, plotVectorField
    from ._plot_methods import plotText, plotPointsWithText
    from ._plot_methods import setScale, plotVectorField



from ._place_holder import Figure_

import os
import json
import sys


def addCustomTemplate(self: Figure_, file_path:str):
    """
    
    """
    
    template = dict()

    # Get the path to the file that called addCustomTemplate()
    namespace = sys._getframe(1).f_globals # caller's globals
    callerPath = namespace['__file__']

    # Get the directory of the file that called addCustomTemplate()
    callerDirectory = "\\".join(callerPath.split("\\")[:-1]) 

    with open(
        callerDirectory + "\\" + file_path + ".json"
    ) as f:
            template:dict
            template = json.load(f)
    
    defaultTemplate = self.templates['default']
    
    for key, value in defaultTemplate.items():
        if(key not in template.keys()):
            template[key] = value

    self.templates[file_path] = template

def getTemplate(file_path:str) -> dict:
    template = dict()
    with open(
        os.path.dirname(__file__) + "/Templates/" + file_path + ".json"
    ) as f:
        template = json.load(f)

    return template


def getAllTemplates() -> dict:
    all_files = os.listdir(os.path.dirname(__file__) + "/Templates/")
    json_files = filter(lambda x: x[-5:] == '.json', all_files)
    try:
        template_list = dict()
        for file in json_files:
            file = file[0:-5]
            new_template = getTemplate(file)
            template_list[file] = new_template
    except ImportError:
        raise ImportError("""Error occured when importing templates, check if
                          everything is setup correctly."""
                          )

    return template_list



from .Chart import Chart
from ._place_holder import Figure_

from ._template_handling import getAllTemplates
import matplotlib.pyplot as plt
import matplotlib.figure
import matplotlib


class Figure(Figure_):
    """Class containing everything needed in order to create complex figures
    with pultiple plots.

    Methods:
    ========

    Add*
    ----

    *   ``addChart``
    *   ``addCustomTemplate``
    *   ``addFigure``

    Set*
    ----

    *   ``setTitle``

    Fig*
    ----

    *   ``figSave``
    *   ``figShow``

    """

    def __init__(self) -> None:
        """Create a new figure.

        Begin by calling the ``addFigure`` method in order to create the
        figure. If you want to, create a custom template by using
        ``addCustomTemplate`` before creating the figure.
        """
        self.charts: dict[str, Chart] = dict()

        self.templates = getAllTemplates()


    from ._template_handling import addCustomTemplate

    from ._set_figure import setTitle, figSave, figShow
    from ._set_figure import addFigure, addChart
    from ._set_figure import addGraph # LEGACY METHOD
    from ._template_handling import addCustomTemplate



from .Chart._place_holder import Chart__

import matplotlib.pyplot as plt
import matplotlib.figure
import matplotlib


class Figure_:
    def __init__(self):
        self.templates: dict[str, dict]
        self.template: dict

        self.fig: matplotlib.figure.Figure

class Chart_(Chart__):
    def __init__(self, fig: Figure_, subPlot: plt.Axes):
        super().__init__()
        
        self.fig: Figure_ = fig
        self.plot: plt.Axes = subPlot
